/*
	TextActivity.java
*/
package org.linphone;

import org.linphone.core.LinphoneCore;
import android.app.Activity;
import android.content.SharedPreferences;
import android.media.AudioManager;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.Editable;
import android.text.TextWatcher;
import android.text.method.ScrollingMovementMethod;
import android.util.TypedValue;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.EditText;

public class TextActivity extends Activity{
	private static TextActivity theText;
	private AutoScrollTextView recieveText;
	private EditText sendText;
	private SendTextWatcher stw;
	
	public void enterInCallState() {
		setVolumeControlStream(AudioManager.STREAM_VOICE_CALL);
	}
	public void exitInCallState() {
		setVolumeControlStream(AudioManager.USE_DEFAULT_STREAM_TYPE);
	}
	
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.text_activity_menu, menu);
		return true;
	}
	
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.menu_mute:
			LinphoneService.instance().getLinphoneCore().muteMic(true);
			return true;
		case R.id.menu_clear_text:
			clear();
			return true;
		case R.id.menu_hangup:
			LinphoneService ls = LinphoneService.instance();
			if (ls.incall()){
				LinphoneCore lLinphoneCore =  ls.getLinphoneCore();
				lLinphoneCore.terminateCall(lLinphoneCore.getCurrentCall());
			}
			return true;
		default:
			return LinphoneActivity.instance().onOptionsItemSelected(item);
		}
	}
		
	public static TextActivity getText(){
		if (theText == null)
			return null;
		return theText;
	}
	
	public void onDestroy(){
		theText = null;
		super.onDestroy();
	}
	
	public void clear(){
		stw.enable(false);
		recieveText.setText("");
		sendText.setText("");
		stw.enable(true);
	}
	
	private void saveDataFromCurrentState() {
		SharedPreferences myPrefs= getSharedPreferences("saved_text", 0);
		SharedPreferences.Editor myEditor= myPrefs.edit();
		myEditor.putString("displayText", recieveText.getText().toString());
		myEditor.putString("sendText", sendText.getText().toString());
		myEditor.commit();
	}
	private void updateFromSavedState() {
		stw.enable(false);
		SharedPreferences myPrefs= getSharedPreferences("saved_text", 0);
		if((myPrefs!= null) && (myPrefs.contains("recieve_text"))) {
			recieveText.setText(myPrefs.getString("recieve_text", ""));
		}
		if((myPrefs!= null) && (myPrefs.contains("send_text"))) {
			sendText.setText(myPrefs.getString("send_text", ""));
			sendText.setSelection(sendText.length());
		}
		stw.enable(true);
	}
		
	public void onPause(){
		super.onPause();
		saveDataFromCurrentState();
	}
	
	public void onResume(){
		super.onResume();
		realtimeTextRecieved(LinphoneService.instance().getLinphoneCore());
		getPreferences();
	}
	
	public void onStart(){
		super.onStart();
		updateFromSavedState();
		if (LinphoneService.instance().incall()) {
			enterInCallState();
		}
		else {
			exitInCallState();
		}
	}
	
	private void getPreferences(){
		SharedPreferences mpref = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
		if (mpref != null) {
			String ssize = mpref.getString("text_size_key", "18");
			int isize = Integer.valueOf(ssize);
			if (isize > 0) {
				sendText.setTextSize(TypedValue.COMPLEX_UNIT_SP,isize);
				recieveText.setTextSize(TypedValue.COMPLEX_UNIT_SP,isize);
			}
		}
	}
			
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.text);	
		sendText=(EditText)findViewById(R.id.send_text);
		sendText.setMovementMethod(new ScrollingMovementMethod());
		sendText.addTextChangedListener(stw = new SendTextWatcher());
		recieveText = (AutoScrollTextView) findViewById(R.id.receive_text);
		recieveText.setMovementMethod(new ScrollingMovementMethod());
		theText = this;
	}
	
	public void realtimeTextRecieved(LinphoneCore lc) {
		String newtext = new String();
		int c;
		int sudd=0;
		
		while ((c = lc.textStreamGetchar32())>0) {
			if (c == 0x08) {
				if (newtext.length() > 0) {
					newtext = newtext.substring(0,newtext.length()-1);
				} else {
					sudd++;
				}
			} else if (c == 0x2028) {
				newtext += "\n";
			} else {
				newtext += (char)c;
			}
		}
		if (sudd > 0) {
			if (recieveText.length()>sudd){
				CharSequence cs = recieveText.getText();
				recieveText.setText(cs.subSequence(0, cs.length()-sudd));
			} else if (recieveText.length()>0){
				recieveText.setText("");
			}
		}
		if (newtext.length()>0) {
			recieveText.append(newtext);
		}

		recieveText.post(new Runnable(){public void run(){
			recieveText.scrollDown();
		}});
	}
	
	private class SendTextWatcher implements TextWatcher {
		private CharSequence bc;
		private boolean enable;
		
		public SendTextWatcher() {
			enable = false;
			bc = null;
		}
		public void enable(boolean e) {
			enable = e;
		}
		
		public void afterTextChanged(Editable s) {}
		public void beforeTextChanged(CharSequence cs, int start, int count,
				int after) {
			if (!enable) return;
			if (count>0) {
				bc = cs.subSequence(start,start+count);
			} else {
				bc = null;
			}
		}
		
		public void onTextChanged(CharSequence cs, int start, int before,
				int count) {
			if (!enable) return;
			LinphoneCore lc=LinphoneService.instance().getLinphoneCore();
			int ic=count;
			int remove;
			int i;
			
			if (bc != null) { 
				remove = bc.length();
			} else {
				remove = 0;
			}

			i=0;
			while (remove>0&&ic>0) {
				if (cs.charAt(start+i)!=bc.charAt(i)) {
					break;
				}
				i++;
				ic--;
				remove--;	
			}
			
			while (remove>0) {
				lc.textStreamPutchar32(0x08);
				remove--;
			}
			
			i=count-ic;
			while (i<count) {
				if (cs.charAt(start+i) == '\n'){
					lc.textStreamPutchar32(0x2028); /* preferred enter in t140 */
				}
				else{
					lc.textStreamPutchar32(cs.charAt(start+i));
				}
				i++;
			}
		}
	}
}
